import 'dart:async';
import 'dart:convert';

import 'package:lori_sdk/data/models/authResponse.dart';
import 'package:shared_preferences/shared_preferences.dart';

class AuthPersistence {
  static const _AUTH_RESPONSE_USER = 'x_auth_response_user';
  static const _AUTH_RESPONSE_CLIENT = 'x_auth_response_client';

  Future<SharedPreferences> open() async =>
      await SharedPreferences.getInstance();

  // NOTE user

  Future<void> inserUser(AuthResponse response) async {
    final pref = await open();
    if (pref.containsKey(_AUTH_RESPONSE_USER))
      pref?.remove(_AUTH_RESPONSE_USER);

    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;

    await pref.setString(_AUTH_RESPONSE_USER, jsonEncode(response.toJson()));
    return;
  }

  Future<AuthResponse> getUser() async {
    final pref = await open();

    String data = pref.getString(_AUTH_RESPONSE_USER);

    AuthResponse _current;

    if (data != null) {
      _current = AuthResponse.fromJson(jsonDecode(data));
    }

    return _current;
  }

  Future<void> removeUser() async {
    final prefs = await open();
    return prefs.remove(_AUTH_RESPONSE_USER);
  }

  // NOTE client

  Future<void> inserClient(AuthResponse response) async {
    final pref = await open();
    if (pref.containsKey(_AUTH_RESPONSE_CLIENT))
      pref?.remove(_AUTH_RESPONSE_CLIENT);

    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;

    await pref.setString(_AUTH_RESPONSE_CLIENT, jsonEncode(response.toJson()));
    return;
  }

  Future<AuthResponse> getClient() async {
    final pref = await open();

    String data = pref.getString(_AUTH_RESPONSE_CLIENT);

    AuthResponse _current;

    if (data != null) {
      _current = AuthResponse.fromJson(jsonDecode(data));
    }

    return _current;
  }

  Future<void> removeClient() async {
    final prefs = await open();
    return prefs.remove(_AUTH_RESPONSE_CLIENT);
  }

  // NOTE all
  Future<void> deleteAll() async {
    final pref = await open();
    await pref.remove(_AUTH_RESPONSE_USER);
    await pref.remove(_AUTH_RESPONSE_CLIENT);
    return;
  }

  Future close() async {}

  // Future<List<AuthResponse>> _all() async {
  //   await open();
  //   String authResponsesInJson = db.getString(_AUTH_RESPONSES_KEY);
  //   List<AuthResponse> responses = [];
  //   if (authResponsesInJson != null) {
  //     List<Map> responsesMap = List.from(jsonDecode(authResponsesInJson));
  //     for (var map in responsesMap) {
  //       responses.add(AuthResponse.fromJson(map));
  //     }
  //   }
  //   return responses;
  // }
}
